<?php

declare(strict_types=1);

namespace Dedi\SyliusQuotePlugin\Calculator;

use Dedi\SyliusQuotePlugin\Entity\QuoteInterface;
use Dedi\SyliusQuotePlugin\Entity\QuoteItemInterface;
use Sylius\Component\Channel\Context\ChannelContextInterface;
use Sylius\Component\Core\Model\ChannelInterface;
use Sylius\Component\Taxation\Calculator\CalculatorInterface;
use Sylius\Component\Taxation\Resolver\TaxRateResolverInterface;

class QuoteTaxesCalculator implements QuoteTaxesCalculatorInterface
{
    private ChannelContextInterface $channelContext;

    private TaxRateResolverInterface $taxRateResolver;

    private CalculatorInterface $calculator;

    public function __construct(
        ChannelContextInterface $channelContext,
        TaxRateResolverInterface $taxRateResolver,
        CalculatorInterface $calculator
    ) {
        $this->channelContext = $channelContext;
        $this->taxRateResolver = $taxRateResolver;
        $this->calculator = $calculator;
    }

    public function getIncludedTax(QuoteInterface $quote): int
    {
        $totalTax = 0;

        /** @var ChannelInterface $channel */
        $channel = $this->channelContext->getChannel();

        /** @var QuoteItemInterface $item */
        foreach ($quote->getItems() as $item) {
            $taxRate = $this->taxRateResolver->resolve($item->getVariant(), ['zone' => $channel->getDefaultTaxZone()]);

            if (null === $taxRate || !$taxRate->isIncludedInPrice()) {
                continue;
            }

            $totalTax += $this->calculator->calculate($item->getTotal(), $taxRate);
        }

        return (int) $totalTax;
    }

    public function getExcludedTax(QuoteInterface $quote): int
    {
        $totalTax = 0;

        /** @var ChannelInterface $channel */
        $channel = $this->channelContext->getChannel();

        /** @var QuoteItemInterface $item */
        foreach ($quote->getItems() as $item) {
            $taxRate = $this->taxRateResolver->resolve($item->getVariant(), ['zone' => $channel->getDefaultTaxZone()]);

            if (null === $taxRate || $taxRate->isIncludedInPrice()) {
                continue;
            }

            $totalTax += $this->calculator->calculate($item->getTotal(), $taxRate);
        }

        return (int) $totalTax;
    }
}
