<?php

declare(strict_types=1);

namespace Dedi\SyliusQuotePlugin\Form\Type;

use Dedi\SyliusQuotePlugin\Entity\QuoteInterface;
use Dedi\SyliusQuotePlugin\Entity\QuoteInternalCommentInterface;
use Sylius\Bundle\MoneyBundle\Form\Type\MoneyType;
use Sylius\Bundle\ResourceBundle\Form\Type\AbstractResourceType;
use Sylius\Component\Core\Model\AdminUserInterface;
use Symfony\Component\Form\Extension\Core\Type\CollectionType;
use Symfony\Component\Form\Extension\Core\Type\DateType;
use Symfony\Component\Form\Extension\Core\Type\TextareaType;
use Symfony\Component\Form\FormBuilderInterface;
use Symfony\Component\Form\FormEvent;
use Symfony\Component\Form\FormEvents;
use Symfony\Component\Security\Core\Security;
use Symfony\Component\Validator\Constraints\NotBlank;

class QuoteType extends AbstractResourceType
{
    private Security $security;

    public function __construct(
        string $dataClass,
        array $validationGroups = [],
        Security $security
    ) {
        parent::__construct($dataClass, $validationGroups);

        $this->security = $security;
    }

    public function buildForm(FormBuilderInterface $builder, array $options): void
    {
        $builder
            ->add('expirationDate', DateType::class, [
                'label' => false,
                'widget' => 'single_text',
                'html5' => true,
            ])
            ->add('informationComment', TextareaType::class, [
                'label' => false,
            ])
            ->add('internalComments', CollectionType::class, [
                'label' => false,
                'entry_type' => QuoteInternalCommentType::class,
                'entry_options' => ['label' => false],
                'allow_add' => true,
                'allow_delete' => true,
                'by_reference' => false,
            ])
            ->add('items', CollectionType::class, [
                'label' => false,
                'entry_type' => QuoteItemType::class,
                'entry_options' => [
                    'label' => false,
                    'currency' => $options['data']->getCurrencyCode(),
                    'quote' => $builder->getData(),
                ],
                'allow_add' => true,
                'allow_delete' => true,
                'by_reference' => false,
                'delete_empty' => true,
            ])
            ->add('total', MoneyType::class, [
                'label' => false,
                'currency' => $options['data']->getCurrencyCode(),
                'constraints' => [
                    new NotBlank(['groups' => $this->validationGroups]),
                ],
            ]);

        $builder->addEventListener(FormEvents::SUBMIT, [$this, 'setInternalCommentsAdminUser']);
    }

    public function setInternalCommentsAdminUser(FormEvent $event): void
    {
        /** @var QuoteInterface $quote */
        $quote = $event->getData();
        if (!$quote instanceof QuoteInterface) {
            throw new \InvalidArgumentException(sprintf(
                'Expected an instance of "%s", got "%s"',
                QuoteInterface::class,
                get_class($quote)
            ));
        }

        $currentUser = $this->security->getUser();
        if (!$currentUser instanceof AdminUserInterface) {
            return;
        }

        /** @var QuoteInternalCommentInterface $internalComment */
        foreach ($quote->getInternalComments() as $internalComment) {
            if (null === $internalComment->getId()) {
                $internalComment->setUser($currentUser);
            }
        }
    }

    public function getBlockPrefix(): string
    {
        return 'dedi_quote_quote';
    }
}
